@@ -1,5 +1,6 @@
 use crate::{
     complex::NumStr::{Matrix, Num, Str, Vector},
+    graph::place_var,
     math::do_math,
     Options,
 };
@@ -9,7 +10,7 @@ use rug::{
         Special::{Infinity, Nan},
     },
     ops::Pow,
-    Complex, Float,
+    Complex, Float, Integer,
 };
 use std::{
     cmp::Ordering,
@@ -427,11 +428,23 @@ pub fn shr(a: &Complex, b: &Complex) -> Complex
 }
 pub fn ne(a: &Complex, b: &Complex) -> Complex
 {
-    Complex::with_val(a.prec(), (a != b) as u8)
+    let c: Complex = a - b.clone();
+    let int = Integer::from(10).pow(a.prec().0 / 4);
+    let re: Float = c.real().clone() * int.clone();
+    let re: Float = re.round() / int.clone();
+    let im: Float = c.imag().clone() * int.clone();
+    let im: Float = im.round() / int;
+    Complex::with_val(a.prec(), (!re.is_zero() || !im.is_zero()) as u8)
 }
 pub fn eq(a: &Complex, b: &Complex) -> Complex
 {
-    Complex::with_val(a.prec(), (a == b) as u8)
+    let c: Complex = a - b.clone();
+    let int = Integer::from(10).pow(a.prec().0 / 4);
+    let re: Float = c.real().clone() * int.clone();
+    let re: Float = re.round() / int.clone();
+    let im: Float = c.imag().clone() * int.clone();
+    let im: Float = im.round() / int;
+    Complex::with_val(a.prec(), (re.is_zero() && im.is_zero()) as u8)
 }
 pub fn ge(a: &Complex, b: &Complex) -> Complex
 {
@@ -728,7 +741,8 @@ pub fn mvec(
                                 }
                             }
                             "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec"
-                            | "mat"
+                            | "mat" | "D" | "integrate" | "arclength" | "area" | "length"
+                            | "slope"|"lim"|"limit"
                                 if i + 2 < func.len() && func[i + 2] == Str(var.to_string()) =>
                             {
                                 sum.push(bracket)
@@ -807,7 +821,8 @@ pub fn mvec(
                                 }
                             }
                             "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec"
-                            | "mat"
+                            | "mat" | "D" | "integrate" | "arclength" | "area" | "length"
+                            | "slope"|"lim"|"limit"
                                 if i + 2 < func.len() && func[i + 2] == Str(var.to_string()) =>
                             {
                                 sum.push(bracket)
@@ -927,6 +942,7 @@ pub fn sum(
                             }
                         }
                         "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec" | "mat"
+                        | "D" | "integrate" | "arclength" | "area" | "length" | "slope"|"lim"|"limit"
                             if i + 2 < func.len() && func[i + 2] == Str(var.to_string()) =>
                         {
                             sum.push(bracket)
@@ -1201,6 +1217,7 @@ pub fn eigenvalues(a: &[Vec<Complex>]) -> Result<Vec<Complex>, &'static str>
                 Complex::with_val(a[0][0].prec(), 1),
                 -a[0][0].clone() - a[1][1].clone(),
                 a[0][0].clone() * a[1][1].clone() - a[0][1].clone() * a[1][0].clone(),
+                false,
             )),
             3 => Ok(cubic(
                 Complex::with_val(a[0][0].prec(), -1),
@@ -1216,6 +1233,7 @@ pub fn eigenvalues(a: &[Vec<Complex>]) -> Result<Vec<Complex>, &'static str>
                     + a[0][1].clone() * a[1][2].clone() * a[2][0].clone()
                     + a[0][2].clone() * a[1][0].clone() * a[2][1].clone()
                     - a[0][2].clone() * a[1][1].clone() * a[2][0].clone(),
+                false,
             )),
             _ => Err("unsupported"),
         }
@@ -1225,7 +1243,7 @@ pub fn eigenvalues(a: &[Vec<Complex>]) -> Result<Vec<Complex>, &'static str>
         Err("not square")
     }
 }
-pub fn quadratic(a: Complex, b: Complex, c: Complex) -> Vec<Complex>
+pub fn quadratic(a: Complex, b: Complex, c: Complex, real: bool) -> Vec<Complex>
 {
     if a.is_zero()
     {
@@ -1235,13 +1253,31 @@ pub fn quadratic(a: Complex, b: Complex, c: Complex) -> Vec<Complex>
     let p: Complex = p - (4 * c * a.clone());
     let p = p.sqrt();
     let a: Complex = 2 * a;
-    vec![(p.clone() - b.clone()) / a.clone(), (-p - b) / a]
+    if real
+    {
+        let z1 = (p.clone() - b.clone()) / a.clone();
+        let z2 = (-p - b) / a;
+        let mut vec = Vec::new();
+        if z1.imag().to_f64().abs() < 0.0000000000000001
+        {
+            vec.push(z1)
+        }
+        if z2.imag().to_f64().abs() < 0.0000000000000001
+        {
+            vec.push(z2)
+        }
+        vec
+    }
+    else
+    {
+        vec![(p.clone() - b.clone()) / a.clone(), (-p - b) / a]
+    }
 }
-pub fn cubic(a: Complex, b: Complex, c: Complex, d: Complex) -> Vec<Complex>
+pub fn cubic(a: Complex, b: Complex, c: Complex, d: Complex, real: bool) -> Vec<Complex>
 {
     if a.is_zero()
     {
-        return quadratic(b, c, d);
+        return quadratic(b, c, d, real);
     }
     let prec = a.prec();
     let threerecip = Float::with_val(prec.0, 3).recip();
@@ -1282,11 +1318,37 @@ pub fn cubic(a: Complex, b: Complex, c: Complex, d: Complex) -> Vec<Complex>
     //-((1 - i sqrt(3)) (-2 b^3 + 3 sqrt(3) sqrt(4 b^3 d - b^2 c^2 - 18 b c d + 4 c^3 + 27 d^2) + 9 b c - 27 d)^(1/3))/(6 2^(1/3)) + ((1 + i sqrt(3)) (3 c - b^2))/(3 2^(2/3) (-2 b^3 + 3 sqrt(3) sqrt(4 b^3 d - b^2 c^2 - 18 b c d + 4 c^3 + 27 d^2) + 9 b c - 27 d)^(1/3)) - b/3
     //-((1 + i sqrt(3)) (-2 b^3 + 3 sqrt(3) sqrt(4 b^3 d - b^2 c^2 - 18 b c d + 4 c^3 + 27 d^2) + 9 b c - 27 d)^(1/3))/(6 2^(1/3)) + ((1 - i sqrt(3)) (3 c - b^2))/(3 2^(2/3) (-2 b^3 + 3 sqrt(3) sqrt(4 b^3 d - b^2 c^2 - 18 b c d + 4 c^3 + 27 d^2) + 9 b c - 27 d)^(1/3)) - b/3
     let omega: Complex = (1 + (Complex::with_val(prec, (0, 1)) * threesqrt.clone())) / 2;
-    vec![
-        (left.clone() - right.clone() - b.clone()) / 3,
-        ((-omega.clone() * left.clone()) + (omega.clone().conj() * right.clone()) - b.clone()) / 3,
-        ((-omega.clone().conj() * left) + (omega * right) - b.clone()) / 3,
-    ]
+    if real
+    {
+        let z1: Complex = (left.clone() - right.clone() - b.clone()) / 3;
+        let z2: Complex =
+            ((-omega.clone() * left.clone()) + (omega.clone().conj() * right.clone()) - b.clone())
+                / 3;
+        let z3: Complex = ((-omega.clone().conj() * left) + (omega * right) - b.clone()) / 3;
+        let mut vec = Vec::new();
+        if z1.imag().to_f64().abs() < 0.0000000000000001
+        {
+            vec.push(z1)
+        }
+        if z2.imag().to_f64().abs() < 0.0000000000000001
+        {
+            vec.push(z2)
+        }
+        if z3.imag().to_f64().abs() < 0.0000000000000001
+        {
+            vec.push(z3)
+        }
+        vec
+    }
+    else
+    {
+        vec![
+            (left.clone() - right.clone() - b.clone()) / 3,
+            ((-omega.clone() * left.clone()) + (omega.clone().conj() * right.clone()) - b.clone())
+                / 3,
+            ((-omega.clone().conj() * left) + (omega * right) - b.clone()) / 3,
+        ]
+    }
 }
 pub fn variance(a: &[Complex], prec: (u32, u32)) -> Complex
 {
@@ -1306,7 +1368,7 @@ pub fn recursion(
 {
     for fv in func_vars.clone()
     {
-        if fv.0.contains('(')
+        if fv.0.ends_with(')')
         {
             if fv.0.contains(',')
             {
@@ -1494,4 +1556,736 @@ pub fn recursion(
         }
     }
     do_math(func, options, func_vars)
+}
+pub fn gcd(mut x: Integer, mut y: Integer) -> Integer
+{
+    while x != y
+    {
+        if x > y
+        {
+            x -= y.clone()
+        }
+        else
+        {
+            y -= x.clone()
+        }
+    }
+    x
+}
+//simpsons rule
+// pub fn incomplete_beta(x: Complex, a: Complex, b: Complex) -> Complex
+// {
+//     let mut last = Complex::new(a.prec());
+//     let i = 12u32;
+//     let mut area = Complex::new(a.prec());
+//     let n: Complex = x.clone() / 2.pow(i);
+//     for k in 1..=2.pow(i)
+//     {
+//         let g: Complex = k * n.clone();
+//         let f: Complex = 1 - g.clone();
+//         let num: Complex = g.pow(a.clone()) * f.pow(b.clone());
+//         let g: Complex = (k * 2 - 1) * n.clone() / 2;
+//         let f: Complex = 1 - g.clone();
+//         let mid: Complex = g.pow(a.clone()) * f.pow(b.clone());
+//         area += (last + 4 * mid + num.clone()) * x.clone() / (3 * 2.pow(i + 1));
+//         last = num;
+//     }
+//     area
+// }
+pub fn incomplete_beta(x: Complex, a: Complex, b: Complex) -> Complex
+{
+    if x.real() > &((a.real().clone() + 1) / (a.real() + b.real().clone() + 2))
+    {
+        (gamma(a.real()) * gamma(b.real()) / gamma(&(a.real() + b.real().clone())))
+            - incomplete_beta(1 - x, b, a)
+    }
+    else
+    {
+        let f: Complex = 1 - x.clone();
+        x.clone().pow(a.clone()) * f.pow(b.clone())
+            / (a.clone() * (1 + incomplete_beta_recursion(x, a, b, 1, 10)))
+    }
+}
+fn incomplete_beta_recursion(x: Complex, a: Complex, b: Complex, iter: usize, max: usize)
+    -> Complex
+{
+    if iter == max
+    {
+        Complex::new(x.prec())
+    }
+    else if iter % 2 == 1
+    {
+        let m = (iter - 1) / 2;
+        (-x.clone() * (a.clone() + m) * (a.clone() + b.clone() + m)
+            / ((a.clone() + (2 * m)) * (a.clone() + (2 * m) + 1)))
+            / (1 + incomplete_beta_recursion(x, a, b, iter + 1, max))
+    }
+    else
+    {
+        let m = iter / 2;
+        (x.clone() * m * (b.clone() - m) / ((a.clone() + (2 * m)) * (a.clone() + (2 * m) - 1)))
+            / (1 + incomplete_beta_recursion(x, a, b, iter + 1, max))
+    }
+}
+pub fn incomplete_gamma(s: Complex, z: Complex) -> Complex
+{
+    // let prec = Float::with_val(z.prec().0, 0.1).pow(z.prec().0 / 2);
+    // let mut last: Complex = incomplete_gamma_recursion(s.clone(), z.clone(), 0, 1);
+    // let mut num = incomplete_gamma_recursion(s.clone(), z.clone(), 0, 2);
+    // for m in 3..100
+    // {
+    //     if (num.clone() - last.clone()).abs().real() > &prec
+    //     {
+    //         last = num.clone();
+    //         num = incomplete_gamma_recursion(s.clone(), z.clone(), 0, m);
+    //     }
+    //     else
+    //     {
+    //         break;
+    //     }
+    // }
+    incomplete_gamma_recursion(s, z, 0, 100)
+}
+fn incomplete_gamma_recursion(s: Complex, z: Complex, iter: usize, max: usize) -> Complex
+{
+    if iter == max
+    {
+        Complex::with_val(s.prec(), 1)
+    }
+    else if iter == 0
+    {
+        (z.clone().pow(s.clone()) / z.clone().exp()) / incomplete_gamma_recursion(s, z, 1, max)
+    }
+    else if iter % 2 == 1
+    {
+        z.clone()
+            + ((iter.div_ceil(2) - s.clone()) / incomplete_gamma_recursion(s, z, iter + 1, max))
+    }
+    else
+    {
+        1 + (iter.div_ceil(2) / incomplete_gamma_recursion(s, z, iter + 1, max))
+    }
+}
+pub fn subfactorial(z: Complex) -> Complex
+{
+    //let prec = Float::with_val(z.prec().0, 0.1).pow(z.prec().0 / 2);
+    // let mut last: Complex = subfactorial_recursion(z.clone(), 0, 1);
+    // let mut num = subfactorial_recursion(z.clone(), 0, 2);
+    // for m in 3..100
+    // {
+    //     if (num.clone() - last.clone()).abs().real() > &prec
+    //     {
+    //         last = num.clone();
+    //         num = subfactorial_recursion(z.clone(), 0, m);
+    //     }
+    //     else
+    //     {
+    //         break;
+    //     }
+    // }
+    subfactorial_recursion(z.clone(), 0, 100)
+        + gamma(&(z.real().clone() + 1)) / Complex::with_val(z.prec(), 1).exp()
+}
+fn subfactorial_recursion(z: Complex, iter: usize, max: usize) -> Complex
+{
+    if iter == max
+    {
+        Complex::with_val(z.prec(), 1)
+    }
+    else if iter == 0
+    {
+        Complex::with_val(z.prec(), -1).pow(z.clone()) / subfactorial_recursion(z, 1, max)
+    }
+    else
+    {
+        (z.clone() + iter + 1) - iter / subfactorial_recursion(z, iter + 1, max)
+    }
+}
+pub fn length(
+    func: Vec<NumStr>,
+    func_vars: Vec<(String, Vec<NumStr>)>,
+    options: Options,
+    var: String,
+    mut start: Complex,
+    mut end: Complex,
+    points: usize,
+) -> Result<Complex, &'static str>
+{
+    if start.real() > end.real()
+    {
+        (start, end) = (end, start)
+    }
+    let delta: Complex = (end - start.clone()) / points;
+    let mut x0: Complex = match slope(
+        func.clone(),
+        func_vars.clone(),
+        options,
+        var.clone(),
+        start.clone(),
+        false,
+    )?
+    {
+        Num(nx0) => 1 + nx0.pow(2),
+        Vector(nx0) => nx0
+            .iter()
+            .map(|n| n.clone().pow(2))
+            .fold(Complex::new(options.prec), |total, x| total + x),
+        _ => return Err("not supported arc length data"),
+    }
+    .sqrt();
+    let mut length = Complex::new(options.prec);
+    let h: Complex = delta.clone() / 4;
+    for _ in 0..points
+    {
+        start += delta.clone();
+        let x1 = slope(
+            func.clone(),
+            func_vars.clone(),
+            options,
+            var.clone(),
+            start.clone() - 3 * h.clone(),
+            false,
+        )?;
+        let x2 = slope(
+            func.clone(),
+            func_vars.clone(),
+            options,
+            var.clone(),
+            start.clone() - 2 * h.clone(),
+            false,
+        )?;
+        let x3 = slope(
+            func.clone(),
+            func_vars.clone(),
+            options,
+            var.clone(),
+            start.clone() - h.clone(),
+            false,
+        )?;
+        let x4 = slope(
+            func.clone(),
+            func_vars.clone(),
+            options,
+            var.clone(),
+            start.clone(),
+            false,
+        )?;
+        match (x1, x2, x3, x4)
+        {
+            (Num(nx1), Num(nx2), Num(nx3), Num(nx4)) =>
+            {
+                let nx1: Complex = 1 + nx1.pow(2);
+                let nx2: Complex = 1 + nx2.pow(2);
+                let nx3: Complex = 1 + nx3.pow(2);
+                let nx4: Complex = 1 + nx4.pow(2);
+                let nx4 = nx4.sqrt();
+                length += 2
+                    * h.clone()
+                    * (7 * (x0 + nx4.clone()) + (12 * nx2.sqrt()) + 32 * (nx1.sqrt() + nx3.sqrt()))
+                    / 45;
+                x0 = nx4
+            }
+            (Vector(nx1), Vector(nx2), Vector(nx3), Vector(nx4)) =>
+            {
+                let nx1: Complex = nx1
+                    .iter()
+                    .map(|n| n.clone().pow(2))
+                    .fold(Complex::new(options.prec), |total, x| total + x)
+                    .sqrt();
+                let nx2: Complex = nx2
+                    .iter()
+                    .map(|n| n.clone().pow(2))
+                    .fold(Complex::new(options.prec), |total, x| total + x)
+                    .sqrt();
+                let nx3: Complex = nx3
+                    .iter()
+                    .map(|n| n.clone().pow(2))
+                    .fold(Complex::new(options.prec), |total, x| total + x)
+                    .sqrt();
+                let nx4 = nx4
+                    .iter()
+                    .map(|n| n.clone().pow(2))
+                    .fold(Complex::new(options.prec), |total, x| total + x)
+                    .sqrt();
+                length +=
+                    2 * h.clone() * (7 * (x0 + nx4.clone()) + (12 * nx2) + 32 * (nx1 + nx3)) / 45;
+                x0 = nx4
+            }
+            (_, _, _, _) => return Err("not supported arc length data"),
+        };
+    }
+    Ok(length)
+}
+#[allow(clippy::too_many_arguments)]
+pub fn area(
+    mut func: Vec<NumStr>,
+    func_vars: Vec<(String, Vec<NumStr>)>,
+    options: Options,
+    var: String,
+    mut start: Complex,
+    end: Complex,
+    points: usize,
+    combine: bool,
+) -> Result<NumStr, &'static str>
+{
+    let mut funcs = Vec::new();
+    if combine && func[0].str_is("{")
+    {
+        let mut brackets = 0;
+        let mut last = 1;
+        for (i, f) in func.iter().enumerate()
+        {
+            if let Str(s) = f
+            {
+                match s.as_str()
+                {
+                    "(" | "{" => brackets += 1,
+                    ")" | "}" => brackets -= 1,
+                    "," if brackets == 1 =>
+                    {
+                        funcs.push(func[last..i].to_vec());
+                        last = i + 1;
+                    }
+                    _ =>
+                    {}
+                }
+            }
+        }
+        if last != 1
+        {
+            func = func[last..func.len() - 1].to_vec();
+        }
+    }
+    let mut areavec: Vec<Complex> = Vec::new();
+    let div = Complex::with_val(options.prec, 0.5).pow(options.prec.0 / 2);
+    let delta: Complex = (end - start.clone()) / points;
+    let mut area: Complex = Complex::new(options.prec);
+    let mut x0 = do_math(
+        place_var(func.clone(), &var, Num(start.clone())),
+        options,
+        func_vars.clone(),
+    )?;
+    if !funcs.is_empty()
+    {
+        let mut nx0t = Complex::new(options.prec);
+        for i in &funcs
+        {
+            nx0t += ((do_math(
+                place_var(i.clone(), &var, Num(start.clone() + div.clone())),
+                options,
+                func_vars.clone(),
+            )?
+            .num()?
+                - do_math(
+                    place_var(i.clone(), &var, Num(start.clone())),
+                    options,
+                    func_vars.clone(),
+                )?
+                .num()?)
+                / div.clone())
+            .pow(2);
+        }
+        x0 = Num(x0.num()? * nx0t.sqrt());
+    }
+    let h: Complex = delta.clone() / 4;
+    for _ in 0..points
+    {
+        start += delta.clone();
+        let x1 = do_math(
+            place_var(func.clone(), &var, Num(start.clone() - 3 * h.clone())),
+            options,
+            func_vars.clone(),
+        )?;
+        let x2 = do_math(
+            place_var(func.clone(), &var, Num(start.clone() - 2 * h.clone())),
+            options,
+            func_vars.clone(),
+        )?;
+        let x3 = do_math(
+            place_var(func.clone(), &var, Num(start.clone() - h.clone())),
+            options,
+            func_vars.clone(),
+        )?;
+        let x4 = do_math(
+            place_var(func.clone(), &var, Num(start.clone())),
+            options,
+            func_vars.clone(),
+        )?;
+        match (x0, x1, x2, x3, x4.clone())
+        {
+            (Num(nx0), Num(nx1), Num(nx2), Num(nx3), Num(nx4)) if funcs.is_empty() =>
+            {
+                area += 2 * h.clone() * (7 * (nx0 + nx4) + 12 * nx2 + 32 * (nx1 + nx3)) / 45;
+                x0 = x4;
+            }
+            (Num(nx0), Num(nx1), Num(nx2), Num(nx3), Num(nx4)) =>
+            {
+                let mut nx1t = Complex::new(options.prec);
+                let mut nx2t = Complex::new(options.prec);
+                let mut nx3t = Complex::new(options.prec);
+                let mut nx4t = Complex::new(options.prec);
+                for i in &funcs
+                {
+                    nx1t += ((do_math(
+                        place_var(i.clone(), &var, Num(start.clone() - 3 * h.clone() + div.clone())),
+                        options,
+                        func_vars.clone(),
+                    )?
+                    .num()?
+                        - do_math(
+                            place_var(i.clone(), &var, Num(start.clone() - 3 * h.clone())),
+                            options,
+                            func_vars.clone(),
+                        )?
+                        .num()?)
+                        / div.clone())
+                    .pow(2);
+                    nx2t += ((do_math(
+                        place_var(
+                            i.clone(),
+                            &var,
+                            Num(start.clone() - 2 * h.clone() + div.clone()),
+                        ),
+                        options,
+                        func_vars.clone(),
+                    )?
+                    .num()?
+                        - do_math(
+                            place_var(i.clone(), &var, Num(start.clone() - 2 * h.clone())),
+                            options,
+                            func_vars.clone(),
+                        )?
+                        .num()?)
+                        / div.clone())
+                    .pow(2);
+                    nx3t += ((do_math(
+                        place_var(
+                            i.clone(),
+                            &var,
+                            Num(start.clone() - h.clone() + div.clone()),
+                        ),
+                        options,
+                        func_vars.clone(),
+                    )?
+                    .num()?
+                        - do_math(
+                            place_var(i.clone(), &var, Num(start.clone() - h.clone())),
+                            options,
+                            func_vars.clone(),
+                        )?
+                        .num()?)
+                        / div.clone())
+                    .pow(2);
+                    nx4t += ((do_math(
+                        place_var(
+                            i.clone(),
+                            &var,
+                            Num(start.clone() + div.clone()),
+                        ),
+                        options,
+                        func_vars.clone(),
+                    )?
+                    .num()?
+                        - do_math(
+                            place_var(i.clone(), &var, Num(start.clone())),
+                            options,
+                            func_vars.clone(),
+                        )?
+                        .num()?)
+                        / div.clone())
+                    .pow(2);
+                }
+                let x4 = nx4 * nx4t.sqrt();
+                area += 2
+                    * h.clone()
+                    * (7 * (nx0 + x4.clone())
+                        + 12 * (nx2 * nx2t.sqrt())
+                        + 32 * ((nx1 * nx1t.sqrt()) + (nx3 * nx3t.sqrt())))
+                    / 45;
+                x0 = Num(x4);
+            }
+            (Vector(nx0), Vector(nx1), Vector(nx2), Vector(nx3), Vector(nx4))
+                if areavec.is_empty() && !combine =>
+            {
+                for i in 0..nx0.len()
+                {
+                    areavec.push(
+                        2 * h.clone()
+                            * (7 * (nx0[i].clone() + nx4[i].clone())
+                                + 12 * nx2[i].clone()
+                                + 32 * (nx1[i].clone() + nx3[i].clone()))
+                            / 45,
+                    )
+                }
+                x0 = x4;
+            }
+            (Vector(nx0), Vector(nx1), Vector(nx2), Vector(nx3), Vector(nx4)) if !combine =>
+            {
+                for (i, v) in areavec.iter_mut().enumerate()
+                {
+                    *v += 2
+                        * h.clone()
+                        * (7 * (nx0[i].clone() + nx4[i].clone())
+                            + 12 * nx2[i].clone()
+                            + 32 * (nx1[i].clone() + nx3[i].clone()))
+                        / 45
+                }
+                x0 = x4;
+            }
+            (_, _, _, _, _) => return Err("not supported area data, if parametric have the 2nd arg start and end with the { } brackets"),
+        }
+    }
+    if areavec.is_empty()
+    {
+        Ok(Num(area))
+    }
+    else
+    {
+        Ok(Vector(areavec))
+    }
+}
+pub fn slope(
+    func: Vec<NumStr>,
+    func_vars: Vec<(String, Vec<NumStr>)>,
+    options: Options,
+    var: String,
+    point: Complex,
+    combine: bool,
+) -> Result<NumStr, &'static str>
+{
+    let h = Complex::with_val(options.prec, 0.5).pow(options.prec.0 / 4);
+    let n1 = do_math(
+        place_var(func.clone(), &var, Num(point.clone())),
+        options,
+        func_vars.clone(),
+    )?;
+    let n2 = do_math(
+        place_var(func.clone(), &var, Num(point + h.clone())),
+        options,
+        func_vars.clone(),
+    )?;
+    match (n1, n2)
+    {
+        (Num(n1), Num(n2)) => Ok(Num((n2 - n1) / h)),
+        (Vector(n1), Vector(n2)) if !combine => Ok(Vector(
+            n2.iter()
+                .zip(n1)
+                .map(|(f, i)| (f - i) / h.clone())
+                .collect::<Vec<Complex>>(),
+        )),
+        (Vector(n1), Vector(n2)) if n1.len() == 1 => Ok(Num((n2[0].clone() - n1[0].clone()) / h)),
+        (Vector(n1), Vector(n2)) if n1.len() == 2 => Ok(Num(
+            (n2[1].clone() - n1[1].clone()) / (n2[0].clone() - n1[0].clone())
+        )),
+        (Vector(n1), Vector(n2)) =>
+        {
+            let num = n2.last().unwrap().clone() - n1.last().unwrap().clone();
+            Ok(Vector(
+                n1[0..n1.len() - 1]
+                    .iter()
+                    .zip(n2[0..n2.len() - 1].to_vec())
+                    .map(|(i, f)| num.clone() / (f - i))
+                    .collect::<Vec<Complex>>(),
+            ))
+        }
+        (_, _) => Err("not supported slope data"),
+    }
+}
+pub fn limit(
+    func: Vec<NumStr>,
+    func_vars: Vec<(String, Vec<NumStr>)>,
+    options: Options,
+    var: String,
+    point: Complex,
+) -> Result<NumStr, &'static str>
+{
+    if point.clone().real().is_infinite()
+    {
+        let (h1,h2);
+        if point.real().is_sign_positive()
+        {
+            h1 = Complex::with_val(options.prec, 2).pow(options.prec.0 / 2);
+            h2 = Complex::with_val(options.prec, 2).pow(options.prec.0 / 2 + 1);
+        }
+        else
+        {
+            h1 = -Complex::with_val(options.prec, 2).pow(options.prec.0 / 2);
+            h2 = -Complex::with_val(options.prec, 2).pow(options.prec.0 / 2 + 1);
+        }
+        let n1 = do_math(
+            place_var(func.clone(), &var, Num(h1)),
+            options,
+            func_vars.clone(),
+        )?;
+        let n2 = do_math(
+            place_var(func.clone(), &var, Num(h2)),
+            options,
+            func_vars.clone(),
+        )?;
+        match (n1, n2)
+        {
+            (Num(n1), Num(n2)) =>
+            {
+                if (n1.clone()-n2).abs().real().clone().ln()<0
+                {
+                    Ok(Num(Complex::with_val(options.prec,n1)))
+                }
+                else if n1.real().is_sign_positive()
+                {
+                    Ok(Num(Complex::with_val(options.prec,Infinity)))
+                }
+                else
+                {
+                    Ok(Num(-Complex::with_val(options.prec,Infinity)))
+                }
+            },
+            (_, _) =>Err("unsupported lim data")
+        }
+    }
+    else
+    {
+        let h1 = Complex::with_val(options.prec, 0.5).pow(options.prec.0 / 2 + 1);
+        let h2 = Complex::with_val(options.prec, 0.5).pow(options.prec.0 / 2);
+        let n1 = do_math(
+            place_var(func.clone(), &var, Num(point.clone() - h1.clone())),
+            options,
+            func_vars.clone(),
+        )?;
+        let n2 = do_math(
+            place_var(func.clone(), &var, Num(point.clone() - h2.clone())),
+            options,
+            func_vars.clone(),
+        )?;
+        let n3 = do_math(
+            place_var(func.clone(), &var, Num(point.clone() + h1)),
+            options,
+            func_vars.clone(),
+        )?;
+        let n4 = do_math(
+            place_var(func.clone(), &var, Num(point + h2)),
+            options,
+            func_vars.clone(),
+        )?;
+        match (n1, n2, n3, n4)
+        {
+            (Num(n1), Num(n2), Num(n3), Num(n4)) =>
+            {
+                if (n1.clone()-n3.clone()).abs().real().clone().ln()<0
+                {
+                    if (n2-n1.clone()).abs().real().clone().ln()<0&&(n4-n3).abs().real().clone().ln()<0
+                    {
+                        Ok(Num(Complex::with_val(options.prec,n1)))
+                    }
+                    else if n1.real().is_sign_positive()
+                    {
+                        Ok(Num(Complex::with_val(options.prec,Infinity)))
+                    }
+                    else
+                    {
+                        Ok(Num(-Complex::with_val(options.prec,Infinity)))
+                    }
+                }
+                else
+                {
+                    Ok(Num(Complex::with_val(options.prec,Nan)))
+                }
+            },
+            (_, _, _, _) =>Err("unsupported lim data")
+        }
+    }
+}
+//https://github.com/IstvanMezo/LambertW-function/blob/master/complex%20Lambert.cpp
+pub fn lambertw(z: Complex, k: isize) -> Complex
+{
+    if z.is_zero()
+    {
+        return if k == 0
+        {
+            Complex::new(z.prec())
+        }
+        else
+        {
+            -Complex::with_val(z.prec(), Infinity)
+        };
+    }
+    if z.imag().is_zero() && (k == 0 || k == -1)
+    {
+        let e: Float = -1 / Float::with_val(z.prec().0, 1).exp();
+        if z.real() == &e
+        {
+            return Complex::with_val(z.prec(), -1);
+        }
+    }
+    let prec = Float::with_val(z.prec().0, 0.1).pow(z.prec().0 / 2);
+    let mut w = initpoint(z.clone(), k);
+    let mut wprev = w.clone();
+    {
+        let zexp = w.clone().exp();
+        let zexpz = w.clone() * zexp.clone();
+        let zexpz_d = zexp.clone() + zexpz.clone();
+        let zexpz_dd = (2 * zexp) + zexpz.clone();
+        w -= 2 * ((zexpz.clone() - z.clone()) * zexpz_d.clone())
+            / ((2 * zexpz_d.pow(2)) - ((zexpz - z.clone()) * zexpz_dd))
+    }
+    for _ in 0..50
+    {
+        if (w.clone() - wprev.clone()).abs().real() > &prec
+        {
+            wprev = w.clone();
+            let zexp = w.clone().exp();
+            let zexpz = w.clone() * zexp.clone();
+            let zexpz_d = zexp.clone() + zexpz.clone();
+            let zexpz_dd = (2 * zexp) + zexpz.clone();
+            w -= 2 * ((zexpz.clone() - z.clone()) * zexpz_d.clone())
+                / ((2 * zexpz_d.pow(2)) - ((zexpz - z.clone()) * zexpz_dd))
+        }
+        else
+        {
+            break;
+        }
+    }
+    w
+}
+fn initpoint(z: Complex, k: isize) -> Complex
+{
+    let pi = Float::with_val(z.prec().0, Pi);
+    let e = Float::with_val(z.prec().0, 1).exp();
+    {
+        let test: Complex = z.clone() + (1 / e.clone());
+        if test.abs().real() <= &1.0001
+        {
+            let p1: Complex = 2 * e * z.clone() + 2;
+            let p = p1.clone().sqrt();
+            if k == 0
+            {
+                return p.clone() - (p1 / 3) + ((11 * p.pow(3)) / 72) - 1;
+            }
+            else if (k == 1 && z.imag() < &0) || (k == -1 && z.imag() > &0)
+            {
+                return -1 - p.clone() - (p1 / 3) - ((11 * p.pow(3)) / 72);
+            }
+        }
+    }
+    {
+        let test: Complex = z.clone() - 0.5;
+        if test.abs().real() <= &0.5001
+        {
+            if k == 0
+            {
+                return (0.35173371 * (0.1237166 + 7.061302897 * z.clone()))
+                    / (2 + 0.827184 * (1 + 2 * z));
+            }
+            else if k == -1
+            {
+                return (Complex::with_val(z.prec(), (-2.2591588985, -4.22096))
+                    * (Complex::with_val(z.prec(), (-14.073271, 33.767687754)) * z.clone()
+                        + Complex::with_val(z.prec(), (-12.7127, 19.071643))
+                            * (1 + 2 * z.clone())))
+                    / (2 + Complex::with_val(z.prec(), (-17.23103, 10.629721)) * (1 + 2 * z));
+            }
+        }
+    }
+    let two_pi_k_i = Complex::with_val(z.prec(), (0, 2 * pi * k));
+    let zln = z.clone().ln() + two_pi_k_i;
+    zln.clone() - zln.ln()
 }
\ No newline at end of file
