@@ -133,6 +133,14 @@ pub fn graph(
                     )
                 }
             }
+            if options.vxr.0 != 0.0 || options.vxr.1 != 0.0
+            {
+                options.xr = options.vxr;
+            }
+            if options.vyr.0 != 0.0 || options.vyr.1 != 0.0
+            {
+                options.yr = options.vyr;
+            }
             let (xticks, yticks) = if options.ticks == -1.0
             {
                 (Some((Fix(1.0), 1)), Some((Fix(1.0), 1)))
@@ -465,6 +473,18 @@ pub fn graph(
                     )
                 }
             }
+            if options.vxr.0 != 0.0 || options.vxr.1 != 0.0
+            {
+                options.xr = options.vxr;
+            }
+            if options.vyr.0 != 0.0 || options.vyr.1 != 0.0
+            {
+                options.yr = options.vyr;
+            }
+            if options.vzr.0 != 0.0 || options.vzr.1 != 0.0
+            {
+                options.zr = options.vzr;
+            }
             let (xticks, yticks, zticks) = if options.ticks == -1.0
             {
                 (
@@ -485,7 +505,40 @@ pub fn graph(
                     Some((Fix((options.zr.1 - options.zr.0) / options.ticks), 1)),
                 )
             };
-            if options.lines || lines
+            if options.surface
+                && points3d[1..6]
+                    .iter()
+                    .all(|p| p[0][2].is_empty() && p[1][2].is_empty())
+                && (points3d[0][0][2].is_empty() || points3d[0][1][2].is_empty())
+            {
+                fg.axes3d()
+                    .set_x_ticks(xticks, &[], &[])
+                    .set_y_ticks(yticks, &[], &[])
+                    .set_z_ticks(zticks, &[], &[])
+                    .set_y_range(Fix(options.yr.0), Fix(options.yr.1))
+                    .set_x_range(Fix(options.xr.0), Fix(options.xr.1))
+                    .set_z_range(Fix(options.zr.0), Fix(options.zr.1))
+                    .set_x_label("x", &[])
+                    .set_y_label("y", &[])
+                    .set_z_label("z", &[])
+                    .lines([0], [0], [0], &[Caption(&re_cap[0]), Color(&colors.re1col)])
+                    .lines([0], [0], [0], &[Caption(&im_cap[0]), Color(&colors.im1col)])
+                    .surface(
+                        &points3d[0][0][2],
+                        options.samples_3d.0 + 1,
+                        options.samples_3d.1 + 1,
+                        Some((options.xr.0, options.yr.0, options.xr.1, options.yr.1)),
+                        &[PointSymbol(options.point_style), Color(&colors.re1col)],
+                    )
+                    .surface(
+                        &points3d[0][1][2],
+                        options.samples_3d.0 + 1,
+                        options.samples_3d.1 + 1,
+                        Some((options.xr.0, options.yr.0, options.xr.1, options.yr.1)),
+                        &[PointSymbol(options.point_style), Color(&colors.re1col)],
+                    );
+            }
+            else if options.lines || lines
             {
                 fg.axes3d()
                     .set_x_ticks(xticks, &[], &[])
@@ -851,12 +904,13 @@ pub fn graph(
         if let Some(time) = watch
         {
             print!("\x1b[G\x1b[K{}ms\n\x1b[G", time.elapsed().as_millis(),);
+            stdout().flush().unwrap();
         }
         if fg.show().is_err()
         {
             print!("\x1b[G\x1b[Kno gnuplot\n\x1b[G{}", prompt(options, &colors));
+            stdout().flush().unwrap();
         }
-        stdout().flush().unwrap();
     })
 }
 #[allow(clippy::type_complexity)]
@@ -884,37 +938,35 @@ pub fn get_list_2d(
     ];
     let den_range = (func.2.xr.1 - func.2.xr.0) / func.2.samples_2d as f64;
     let mut zero = (false, false);
-    let list = func.0.iter().any(|c| c.str_is("±"))
-        || func.1.iter().any(|c| c.1.iter().any(|c| c.str_is("±")));
+    let list = func.0.iter().any(|c| {
+        if let Str(s) = c
+        {
+            matches!(s.as_str(), "±" | "cubic" | "quadratic")
+        }
+        else
+        {
+            false
+        }
+    }) || func.1.iter().any(|c| {
+        c.1.iter().any(|c| {
+            if let Str(s) = c
+            {
+                matches!(s.as_str(), "±" | "cubic" | "quadratic")
+            }
+            else
+            {
+                false
+            }
+        })
+    });
     for i in 0..=func.2.samples_2d
     {
         let n = func.2.xr.0 + i as f64 * den_range;
         let num = Num(Complex::with_val(func.2.prec, n));
         match do_math(
-            func.0
-                .iter()
-                .map(|i| match i
-                {
-                    Str(s) if s == "x" || s == "y" => num.clone(),
-                    _ => i.clone(),
-                })
-                .collect(),
+            place_varxy(func.0.clone(), num.clone()),
             func.2,
-            func.1
-                .iter()
-                .map(|i| {
-                    (
-                        i.0.clone(),
-                        i.1.iter()
-                            .map(|i| match i
-                            {
-                                Str(s) if s == "x" || s == "y" => num.clone(),
-                                _ => i.clone(),
-                            })
-                            .collect(),
-                    )
-                })
-                .collect(),
+            place_funcvarxy(func.1.clone(), num),
         )
         {
             Ok(Num(num)) =>
@@ -1121,67 +1173,42 @@ pub fn get_list_3d(
     let mut modified: Vec<NumStr>;
     let mut modifiedvars: Vec<(String, Vec<NumStr>)>;
     let mut zero = (false, false);
-    let list = func.0.iter().any(|c| c.str_is("±"))
-        || func.1.iter().any(|c| c.1.iter().any(|c| c.str_is("±")));
+    let list = func.0.iter().any(|c| {
+        if let Str(s) = c
+        {
+            matches!(s.as_str(), "±" | "cubic" | "quadratic")
+        }
+        else
+        {
+            false
+        }
+    }) || func.1.iter().any(|c| {
+        c.1.iter().any(|c| {
+            if let Str(s) = c
+            {
+                matches!(s.as_str(), "±" | "cubic" | "quadratic")
+            }
+            else
+            {
+                false
+            }
+        })
+    });
     let mut d2 = false;
     for i in 0..=func.2.samples_3d.0
     {
         let n = func.2.xr.0 + i as f64 * den_x_range;
         let num = Num(Complex::with_val(func.2.prec, n));
-        modified = func
-            .0
-            .iter()
-            .map(|i| match i
-            {
-                Str(s) if s == "x" => num.clone(),
-                _ => i.clone(),
-            })
-            .collect();
-        modifiedvars = func
-            .1
-            .iter()
-            .map(|i| {
-                (
-                    i.0.clone(),
-                    i.1.iter()
-                        .map(|i| match i
-                        {
-                            Str(s) if s == "x" => num.clone(),
-                            _ => i.clone(),
-                        })
-                        .collect(),
-                )
-            })
-            .collect();
+        modified = place_var(func.0.clone(), "x", num.clone());
+        modifiedvars = place_funcvar(func.1.clone(), "x", num);
         for g in 0..=func.2.samples_3d.1
         {
             let f = func.2.yr.0 + g as f64 * den_y_range;
             let num = Num(Complex::with_val(func.2.prec, f));
             match do_math(
-                modified
-                    .iter()
-                    .map(|j| match j
-                    {
-                        Str(s) if s == "y" => num.clone(),
-                        _ => j.clone(),
-                    })
-                    .collect(),
+                place_var(modified.clone(), "y", num.clone()),
                 func.2,
-                modifiedvars
-                    .iter()
-                    .map(|i| {
-                        (
-                            i.0.clone(),
-                            i.1.iter()
-                                .map(|i| match i
-                                {
-                                    Str(s) if s == "y" => num.clone(),
-                                    _ => i.clone(),
-                                })
-                                .collect(),
-                        )
-                    })
-                    .collect(),
+                place_funcvar(modifiedvars.clone(), "y", num),
             )
             {
                 Ok(Num(num)) =>
@@ -1359,6 +1386,75 @@ pub fn get_list_3d(
     }
     (data, zero, d2)
 }
+fn place_funcvarxy(
+    mut funcvar: Vec<(String, Vec<NumStr>)>,
+    num: NumStr,
+) -> Vec<(String, Vec<NumStr>)>
+{
+    for i in funcvar.iter_mut()
+    {
+        for j in i.1.iter_mut()
+        {
+            if let Str(s) = j
+            {
+                if matches!(s.as_str(), "x" | "y")
+                {
+                    *j = num.clone()
+                }
+            }
+        }
+    }
+    funcvar
+}
+fn place_funcvar(
+    mut funcvar: Vec<(String, Vec<NumStr>)>,
+    var: &str,
+    num: NumStr,
+) -> Vec<(String, Vec<NumStr>)>
+{
+    for i in funcvar.iter_mut()
+    {
+        for j in i.1.iter_mut()
+        {
+            if let Str(s) = j
+            {
+                if s == var
+                {
+                    *j = num.clone()
+                }
+            }
+        }
+    }
+    funcvar
+}
+fn place_varxy(mut func: Vec<NumStr>, num: NumStr) -> Vec<NumStr>
+{
+    for i in func.iter_mut()
+    {
+        if let Str(s) = i
+        {
+            if matches!(s.as_str(), "x" | "y")
+            {
+                *i = num.clone()
+            }
+        }
+    }
+    func
+}
+pub fn place_var(mut func: Vec<NumStr>, var: &str, num: NumStr) -> Vec<NumStr>
+{
+    for i in func.iter_mut()
+    {
+        if let Str(s) = i
+        {
+            if s == var
+            {
+                *i = num.clone()
+            }
+        }
+    }
+    func
+}
 fn fail(options: Options, colors: &Colors)
 {
     print!(
@@ -1453,6 +1549,32 @@ fn get_data(
                                 points3d[1].iter().flatten().any(|a| *a != 0.0),
                             );
                         }
+                        2 if func.0.iter().any(|c| c.str_is("±")) =>
+                        {
+                            lines = false;
+                            d2_or_d3.0 = true;
+                            (points2d, _, _) =
+                                get_list_2d((vec![Num(v[0].clone())], Vec::new(), func.2));
+                            let points2dtemp: [[Vec<f64>; 2]; 2];
+                            (points2dtemp, _, re_or_im) =
+                                get_list_2d((vec![Num(v[1].clone())], Vec::new(), func.2));
+                            points2d[0][0].extend(points2dtemp[0][0].clone());
+                            points2d[0][1].extend(points2dtemp[0][1].clone());
+                            points2d[1][0].extend(points2dtemp[1][0].clone());
+                            points2d[1][1].extend(points2dtemp[1][1].clone());
+                            if points2d[0][1].is_empty() && points2d[1][1].is_empty()
+                            {
+                                fail(func.2, &colors);
+                                return (
+                                    (false, false),
+                                    (false, false),
+                                    false,
+                                    true,
+                                    Default::default(),
+                                    Default::default(),
+                                );
+                            }
+                        }
                         2 =>
                         {
                             d2_or_d3.0 = true;
