@@ -1,6 +1,6 @@
 pub fn help()
 {
-    println!(
+    print!(
              "Usage: kalc [FLAGS] function_1 function_2 function_3...\n\x1b[G\
     FLAGS: --help (this message)\n\x1b[G\
     --tau fractions are shown in tau instead of pi\n\x1b[G\
@@ -12,6 +12,10 @@ pub fn help()
     --xr=[min],[max] x range for graphing\n\x1b[G\
     --yr=[min],[max] y range for graphing\n\x1b[G\
     --zr=[min],[max] z range for graphing\n\x1b[G\
+    --vxr=[min],[max] x range for graphing, graph view override, useful for parametric\n\x1b[G\
+    --vyr=[min],[max] y range for graphing, graph view override, useful for parametric\n\x1b[G\
+    --vzr=[min],[max] z range for graphing, graph view override, useful for parametric\n\x1b[G\
+    --vrange=[num] sets all ranges to [-num],[num], graph view override, useful for parametric\n\x1b[G\
     --range=[num] sets all ranges to [-num],[num]\n\x1b[G\
     --point [char] point style for graphing\n\x1b[G\
     --sci toggles scientific notation\n\x1b[G\
@@ -71,23 +75,29 @@ pub fn help()
     - csc, sec, cot, acsc, asec, acot\n\x1b[G\
     - sinh, cosh, tanh, asinh, acosh, atanh\n\x1b[G\
     - csch, sech, coth, acsch, asech, acoth\n\x1b[G\
-    - sqrt, cbrt, square, cube, quadratic(a,b,c) cubic(a,b,c,d)\n\x1b[G\
-    - ln, log(base,num), root(base,exp), sum(var,func,start,end), prod(var,func,start,end)\n\x1b[G\
+    - sqrt, cbrt, square, cube, quadratic(a,b,c) cubic(a,b,c,d) (quadratic and cubic finds the zeros for the given polynomials, you can use cubic(a,b,c,d,1) to only find real solutions for cubics/quadratics)\n\x1b[G\
+    - ln, log(base,num), W(k,z) (product log, branch k, defaults to k=0)\n\x1b[G\
+    - root(base,exp), sum(var,func,start,end), prod(var,func,start,end)\n\x1b[G\
     - abs, sgn, arg\n\x1b[G\
     - ceil, floor, round, int, frac\n\x1b[G\
-    - fact(real), doublefact(real), subfact(natural)\n\x1b[G\
+    - fact, doublefact, subfact (all real)\n\x1b[G\
     - sinc, cis, exp\n\x1b[G\
-    - zeta, gamma, erf, erfc, digamma, ai, binomial/bi/C, P(n,r) (all real only)\n\x1b[G\
-    - deg, rad, grad (all real only)\n\x1b[G\
+    - zeta, gamma, beta, erf, erfc, digamma, ai, multinomial, binomial/bi/C, P(n,r) (all real only)\n\x1b[G\
+    - deg, rad, grad\n\x1b[G\
     - re, im, split(x+yi={{x,y}})\n\x1b[G\
-    - factors, prime, isprime\n\x1b[G\
-    - slog(a,b)\n\x1b[G\
+    - factors, prime, isprime, gcd, lcm\n\x1b[G\
+    - slog(a,b), ssrt(k,a) (k is lambert w branch)\n\x1b[G\
     - piecewise({{value,cond}},{{value2,cond2}}...) (when first condition is met from left to right. value elsewards is nan)\n\x1b[G\
     - vec(var,func,start,end) mat(var,func,start,end) (makes a vector/matrix) start..end is a shortcut to vec(n,n,start,end)\n\x1b[G\
     - tofreq{{a,b,c...}}, tolist{{{{a,b}},{{c,d}}...}} (sorts and counts how many time each number occurs, tolist takes that kind of data and reverses it)\n\x1b[G\
     - variance/var, standarddeviation/σ (sample-bias corrected)\n\x1b[G\
     - percentile({{vec}},nth) (gets number at nth percentile), percentilerank({{vec}},x) (gets percentile rank for x point), quartiles{{vec}} (gets quartiles for data set)\n\x1b[G\
-    - normP(μ,σ,x) (normal distribution pdf) normD(z) (area under curve to the left of z score cdf)\n\n\x1b[G\
+    - normP(μ,σ,x) (normal distribution pdf) normD(z) (area under curve to the left of z score cdf)\n\x1b[G\
+    - betaP(α,β,x) (beta distribution pdf) I(x,a,b) (regularized incomplete beta function, or beta distributions cdf)\n\x1b[G\
+    - roll{{a,b,c...}} rolls die dice{{a,b,c...}} gets the frequency data any amount of different sided die, where a/b/c are number of faces for each die, both also accept {{{{first_dice_face,# of die}},{{second_dice_face,# of die}}...}}\n\x1b[G\
+    - lim(x,f(x),point)\n\x1b[G\
+    - slope(x,f(x),point), can add a 0 to the args to not combine the x and y slopes for parametric equations, and for area\n\x1b[G\
+    - area(x,f(x),from,to (,amount of data points) (,0) ), length(x,f(x),from,to (,amount of data points) ) (bracketed means optional)\n\n\x1b[G\
     Vector operations/functions:\n\x1b[G\
     - dot({{vec1}},{{vec2}}), cross({{vec1}},{{vec2}}), proj/project({{vec1}},{{vec2}})\n\x1b[G\
     - angle({{vec1}},{{vec2}})\n\x1b[G\
@@ -107,7 +117,7 @@ pub fn help()
     - abs, norm\n\x1b[G\
     - len, wid\n\x1b[G\
     - max, min, mean, mode\n\x1b[G\
-    - I(n) produces n identity matrix\n\x1b[G\
+    - iden(n) produces an n dimension identity matrix\n\x1b[G\
     - rotate(theta) produces a rotational matrix\n\x1b[G\
     - other functions are applied like sqrt{{{{2,4}},{{5,6}}}}={{{{sqrt(2),sqrt(4)}},{{sqrt(5),sqrt(6)}}}}\n\n\x1b[G\
     Constants:\n\x1b[G\
@@ -136,6 +146,6 @@ pub fn help()
     r=>ρ, R=>Ρ, s=>σ, S=>Σ, t=>τ, T=>Τ, u=>υ, U=>Υ,\n\x1b[G\
     w=>ω, W=>Ω, y=>ψ, Y=>Ψ, x=>χ, X=>Χ, z=>ζ, Z=>Ζ,\n\x1b[G\
     ==>±, `=>ⁱ _=>∞\n\x1b[G\
-    numbers/minus sign convert to superscript acting as exponents"
+    numbers/minus sign convert to superscript acting as exponents\n\x1b[G"
     );
 }
\ No newline at end of file
