@@ -1,9 +1,8 @@
 use crate::{
-    complex::NumStr::Num, math::do_math, misc::prompt, options::set_commands, parse::input_var,
-    Colors, Options, Variable,
+    complex::NumStr::Num, math::do_math, options::set_commands, parse::input_var, Colors, Options,
+    Variable,
 };
 use rug::{float::Constant::Pi, ops::CompleteRound, Complex, Float};
-use std::io::{Stdout, Write};
 pub fn get_file_vars(
     options: Options,
     vars: &mut Vec<Variable>,
@@ -25,28 +24,43 @@ pub fn get_file_vars(
             let l = split.next().unwrap().to_string();
             let left = if l.contains('(')
             {
-                l.split('(').next().unwrap().to_owned() + "("
+                l.split('(').next().unwrap().to_owned()
             }
             else
             {
                 l.clone()
             };
-            if r.contains(&left) && !blacklist.contains(&left)
+            if if l.contains('(')
+            {
+                r.contains(&(left.clone() + "("))
+                    || r.contains(&(left.clone() + "{"))
+                    || r.contains(&(left.clone() + "["))
+            }
+            else
+            {
+                r.contains(&left)
+            } && !blacklist.contains(&l)
             {
-                blacklist.push(left);
                 if let Some(r) = split.next()
                 {
+                    blacklist.push(l.clone());
                     let l = l.chars().collect::<Vec<char>>();
                     get_file_vars(options, vars, lines.clone(), r, blacklist);
                     for (i, j) in vars.clone().iter().enumerate()
                     {
                         if j.name.len() <= l.len()
                         {
-                            add_var(l, r, i, vars, options, false, false);
+                            if let Err(s) = add_var(l, r, i, vars, options, false, false, false)
+                            {
+                                println!("{}", s)
+                            }
                             continue 'lower;
                         }
                     }
-                    add_var(l, r, 0, vars, options, false, false);
+                    if let Err(s) = add_var(l, r, 0, vars, options, false, false, false)
+                    {
+                        println!("{}", s)
+                    }
                 }
             }
         }
@@ -687,6 +701,7 @@ pub fn get_vars(options: Options) -> Vec<Variable>
         },
     ]
 }
+#[allow(clippy::too_many_arguments)]
 pub fn add_var(
     l: Vec<char>,
     r: &str,
@@ -695,80 +710,72 @@ pub fn add_var(
     options: Options,
     redef: bool,
     replace: bool,
-)
+    null: bool,
+) -> Result<(), &'static str>
 {
-    let mut func_vars: Vec<(isize, String)> = Vec::new();
-    if l.contains(&'(')
+    if null
     {
-        let mut l = l.clone();
-        l.drain(0..=l.iter().position(|c| c == &'(').unwrap());
-        l.pop();
-        for i in l.split(|c| c == &',')
-        {
-            func_vars.push((-1, i.iter().collect()));
-        }
-    }
-    let mut k = 0;
-    for (j, v) in vars.iter().enumerate()
-    {
-        if v.name.len() <= l.len()
-        {
-            k = j;
-            break;
-        }
-    }
-    vars.insert(
-        k,
-        Variable {
-            name: l.clone(),
-            parsed: Vec::new(),
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        },
-    );
-    let parsed = match input_var(
-        r,
-        vars.clone(),
-        &mut func_vars,
-        &mut 0,
-        options,
-        false,
-        &mut (false, 0, 0),
-        false,
-        0,
-        l.clone(),
-    )
-    {
-        Ok(n) => (n.0, n.1),
-        Err(s) =>
-        {
-            println!("failed at: {} {}\x1b[G", l.iter().collect::<String>(), s);
-            return;
-        }
-    };
-    vars.remove(k);
-    if replace
-    {
-        vars[i] = Variable {
-            name: l.clone(),
-            parsed: if l.contains(&'(')
-            {
-                parsed.0
-            }
-            else
-            {
-                vec![do_math(parsed.0, options, parsed.1.clone())
-                    .unwrap_or(Num(Complex::new(options.prec)))]
-            },
-            unparsed: r.to_string(),
-            funcvars: parsed.1,
-        };
+        vars.remove(i);
     }
     else
     {
+        let mut func_vars: Vec<(isize, String)> = Vec::new();
+        if l.contains(&'(')
+        {
+            let mut l = l.clone();
+            if l.drain(0..=l.iter().position(|c| c == &'(').unwrap())
+                .collect::<String>()
+                .contains(',')
+            {
+                return Err("bad var name");
+            }
+            l.pop();
+            for i in l.split(|c| c == &',')
+            {
+                func_vars.push((-1, i.iter().collect()));
+            }
+        }
+        else if l.contains(&',')
+        {
+            return Err("bad var name");
+        }
+        let mut k = 0;
+        for (j, v) in vars.iter().enumerate()
+        {
+            if v.name.len() <= l.len()
+            {
+                k = j;
+                break;
+            }
+        }
         vars.insert(
-            i,
+            k,
             Variable {
+                name: l.clone(),
+                parsed: Vec::new(),
+                unparsed: String::new(),
+                funcvars: Vec::new(),
+            },
+        );
+        let parsed = input_var(
+            r,
+            vars.clone(),
+            &mut func_vars,
+            &mut 0,
+            options,
+            false,
+            false,
+            0,
+            l.clone(),
+        )?;
+        vars.remove(k);
+        if parsed.0.is_empty()
+        {
+            return Err("bad input");
+        }
+        else if replace
+        {
+            vars[i] = Variable {
                 name: l.clone(),
                 parsed: if l.contains(&'(')
                 {
@@ -776,20 +783,38 @@ pub fn add_var(
                 }
                 else
                 {
-                    vec![do_math(parsed.0, options, parsed.1.clone())
-                        .unwrap_or(Num(Complex::new(options.prec)))]
+                    vec![do_math(parsed.0, options, parsed.1.clone())?]
                 },
                 unparsed: r.to_string(),
                 funcvars: parsed.1,
-            },
-        )
-    }
-    if l.contains(&'(') && r.contains("piecewise")
-    {
-        let parsed = vars[i].parsed.clone();
-        vars[i]
-            .funcvars
-            .push((l.iter().collect::<String>(), parsed))
+            };
+        }
+        else
+        {
+            vars.insert(
+                i,
+                Variable {
+                    name: l.clone(),
+                    parsed: if l.contains(&'(')
+                    {
+                        parsed.0
+                    }
+                    else
+                    {
+                        vec![do_math(parsed.0, options, parsed.1.clone())?]
+                    },
+                    unparsed: r.to_string(),
+                    funcvars: parsed.1,
+                },
+            )
+        }
+        if l.contains(&'(') && r.contains("piecewise")
+        {
+            let parsed = vars[i].parsed.clone();
+            vars[i]
+                .funcvars
+                .push((l.iter().collect::<String>(), parsed))
+        }
     }
     if redef
     {
@@ -820,58 +845,48 @@ pub fn add_var(
                             func_vars.push((-1, i.iter().collect::<String>()));
                         }
                     }
-                    let parsed = match input_var(
+                    let parsed = input_var(
                         &v.unparsed.clone(),
                         vars.clone(),
                         &mut func_vars,
                         &mut 0,
                         options,
                         false,
-                        &mut (false, 0, 0),
                         false,
                         0,
                         v.name.clone(),
-                    )
-                    {
-                        Ok(n) => (n.0, n.1),
-                        Err(s) =>
-                        {
-                            println!(
-                                "failed at: {} {}\x1b[G",
-                                v.name.iter().collect::<String>(),
-                                s
-                            );
-                            return;
-                        }
-                    };
-                    let check = (vars[j].parsed.clone(), vars[j].funcvars.clone());
+                    )?;
                     if v.name.contains(&'(')
                     {
-                        (vars[j].parsed, vars[j].funcvars) = parsed.clone();
+                        if vars[j].parsed.clone() != parsed.0
+                            || vars[j].funcvars.clone() != parsed.1
+                        {
+                            redef.push(v.name.clone());
+                            vars[j].parsed = parsed.0.clone();
+                            vars[j].funcvars = parsed.1.clone();
+                        }
                     }
-                    else
+                    else if let Ok(n) = do_math(parsed.0.clone(), options, parsed.1.clone())
                     {
-                        vars[j].parsed = vec![do_math(parsed.0.clone(), options, parsed.1.clone())
-                            .unwrap_or(Num(Complex::new(options.prec)))];
-                    }
-                    if check != parsed
-                    {
-                        redef.push(v.name.clone());
+                        if n != vars[j].parsed[0]
+                        {
+                            redef.push(v.name.clone());
+                            vars[j].parsed = vec![n];
+                        }
                     }
                 }
             }
             k += 1;
         }
     }
+    Ok(())
 }
 pub fn set_commands_or_vars(
     colors: &mut Colors,
     options: &mut Options,
-    stdout: Option<&mut Stdout>,
     vars: &mut Vec<Variable>,
-    cli: bool,
     input: &[char],
-)
+) -> Result<(), &'static str>
 {
     let n = input.iter().collect::<String>();
     let mut split = n.splitn(2, '=');
@@ -881,63 +896,16 @@ pub fn set_commands_or_vars(
     if l.is_empty()
         || l.chars()
             .any(|c| !c.is_alphanumeric() && !matches!(c, '(' | ')' | ',' | '\'' | '`'))
-        || match set_commands(options, colors, vars, &l, r)
-        {
-            Err(s) =>
-            {
-                if !s.is_empty()
-                {
-                    if let Some(stdout) = stdout
-                    {
-                        if s == " "
-                        {
-                            print!(
-                                "\x1b[G\x1b[Kempty input\n\x1b[G\x1b[K{}{}",
-                                prompt(*options, colors),
-                                if options.color { "\x1b[0m" } else { "" },
-                            );
-                            stdout.flush().unwrap()
-                        }
-                        else
-                        {
-                            print!(
-                                "\x1b[G\x1b[K{}\n\x1b[G\x1b[K{}{}",
-                                s,
-                                prompt(*options, colors),
-                                if options.color { "\x1b[0m" } else { "" },
-                            );
-                            stdout.flush().unwrap()
-                        }
-                        if !cli
-                        {
-                            print!(
-                                "{}{}",
-                                prompt(*options, colors),
-                                if options.color { "\x1b[0m" } else { "" }
-                            );
-                            stdout.flush().unwrap()
-                        }
-                        return;
-                    }
-                }
-                true
-            }
-            _ => false,
-        }
     {
-        if let Some(stdout) = stdout
+        return Ok(());
+    }
+    else if let Err(s) = set_commands(options, colors, vars, &l, r)
+    {
+        if s.is_empty()
         {
-            if !cli
-            {
-                print!(
-                    "{}{}",
-                    prompt(*options, colors),
-                    if options.color { "\x1b[0m" } else { "" }
-                );
-                stdout.flush().unwrap()
-            }
+            return Ok(());
         }
-        return;
+        return Err(s);
     }
     let l = l.chars().collect::<Vec<char>>();
     for (i, v) in vars.iter().enumerate()
@@ -949,58 +917,23 @@ pub fn set_commands_or_vars(
         {
             if r == "null"
             {
-                vars.remove(i);
+                add_var(l, r, i, vars, *options, true, true, true)?
             }
             else
             {
-                add_var(l, r, i, vars, *options, true, true);
+                add_var(l, r, i, vars, *options, true, true, false)?
             }
-            if let Some(stdout) = stdout
-            {
-                if !cli
-                {
-                    print!(
-                        "{}{}",
-                        prompt(*options, colors),
-                        if options.color { "\x1b[0m" } else { "" }
-                    );
-                    stdout.flush().unwrap();
-                }
-            }
-            return;
+            return Ok(());
         }
     }
     for (i, j) in vars.iter().enumerate()
     {
         if j.name.len() <= l.len()
         {
-            add_var(l, r, i, vars, *options, true, false);
-            if let Some(stdout) = stdout
-            {
-                if !cli
-                {
-                    print!(
-                        "{}{}",
-                        prompt(*options, colors),
-                        if options.color { "\x1b[0m" } else { "" }
-                    );
-                    stdout.flush().unwrap();
-                }
-            }
-            return;
-        }
-    }
-    add_var(l, r, 0, vars, *options, true, false);
-    if let Some(stdout) = stdout
-    {
-        if !cli
-        {
-            print!(
-                "{}{}",
-                prompt(*options, colors),
-                if options.color { "\x1b[0m" } else { "" }
-            );
-            stdout.flush().unwrap();
+            add_var(l, r, i, vars, *options, true, false, false)?;
+            return Ok(());
         }
     }
+    add_var(l, r, 0, vars, *options, true, false, false)?;
+    Ok(())
 }
\ No newline at end of file
