@@ -30,15 +30,11 @@ use std::{
     thread::JoinHandle,
     time::Instant,
 };
-//figure out probability of winning for 3 turned game
-//lambert w function
+//derivitives and integrals
+//surface area of a 3d curve
+//rpn
 //matrix exponentiation
-//beta distribution
-//support units properly
-//rpn maybe
-//make dice function
-//add view option
-//tangent surface
+//support units properly, add a part to the Num struct where it just stores the unit which then can be dealt with in complex or smth
 #[derive(Clone)]
 pub struct Variable
 {
@@ -128,6 +124,9 @@ pub struct Options
     xr: (f64, f64),
     yr: (f64, f64),
     zr: (f64, f64),
+    vxr: (f64, f64),
+    vyr: (f64, f64),
+    vzr: (f64, f64),
     samples_2d: usize,
     samples_3d: (usize, usize),
     point_style: char,
@@ -142,6 +141,8 @@ pub struct Options
     graph: bool,
     slowcheck: u128,
     var_multiply: bool,
+    interactive: bool,
+    surface: bool,
 }
 impl Default for Options
 {
@@ -166,6 +167,9 @@ impl Default for Options
             xr: (-10.0, 10.0),
             yr: (-10.0, 10.0),
             zr: (-10.0, 10.0),
+            vxr: (0.0, 0.0),
+            vyr: (0.0, 0.0),
+            vzr: (0.0, 0.0),
             samples_2d: 20000,
             samples_3d: (300, 300),
             point_style: '.',
@@ -180,6 +184,8 @@ impl Default for Options
             graph: true,
             slowcheck: 250,
             var_multiply: false,
+            interactive: true,
+            surface: false,
         }
     }
 }
@@ -207,8 +213,25 @@ fn main()
             std::process::exit(1);
         }
     }
+    if !stdin().is_terminal()
+    {
+        for line in stdin().lock().lines()
+        {
+            if !line.as_ref().unwrap().is_empty()
+            {
+                if let Ok(line) = line
+                {
+                    if !line.starts_with('#')
+                    {
+                        args.push(line);
+                    }
+                }
+            }
+        }
+    }
+    options.interactive = args.is_empty();
     let mut stdout = stdout();
-    if args.is_empty()
+    if options.interactive
     {
         terminal::enable_raw_mode().unwrap();
         print!(
@@ -227,16 +250,6 @@ fn main()
             arg.pop();
         }
     }
-    if !stdin().is_terminal()
-    {
-        for line in stdin().lock().lines()
-        {
-            if !line.as_ref().unwrap().is_empty()
-            {
-                args.push(line.unwrap());
-            }
-        }
-    }
     #[cfg(unix)]
     let file_path = &(var("HOME").unwrap() + "/.config/kalc.vars");
     #[cfg(not(unix))]
@@ -244,7 +257,7 @@ fn main()
         "C:\\Users\\{}\\AppData\\Roaming\\kalc.vars",
         var("USERNAME").unwrap()
     );
-    let mut vars: Vec<Variable> = if options.allow_vars && args.is_empty()
+    let mut vars: Vec<Variable> = if options.allow_vars && options.interactive
     {
         get_vars(options)
     }
@@ -252,7 +265,7 @@ fn main()
     {
         Vec::new()
     };
-    if !args.is_empty() && options.allow_vars
+    if !options.interactive && options.allow_vars
     {
         get_cli_vars(options, args.concat(), &mut vars)
     }
@@ -277,63 +290,101 @@ fn main()
                     .collect::<Vec<String>>();
                 let mut split;
                 let args = args.concat();
+                let mut blacklist = if options.interactive
+                {
+                    Vec::new()
+                }
+                else
+                {
+                    vars.iter()
+                        .map(|v| v.name.iter().collect::<String>())
+                        .collect::<Vec<String>>()
+                };
                 'upper: for i in lines.clone()
                 {
                     split = i.splitn(2, '=');
                     let l = split.next().unwrap().to_string();
                     let left = if l.contains('(')
                     {
-                        l.split('(').next().unwrap().to_owned() + "("
+                        l.split('(').next().unwrap().to_owned()
                     }
                     else
                     {
                         l.clone()
                     };
-                    if !(l.starts_with('#') || (!args.is_empty() && !args.contains(&left)))
+                    if options.interactive
+                        || (!blacklist.contains(&l)
+                            && if l.contains('(')
+                            {
+                                args.contains(&(left.clone() + "("))
+                                    || args.contains(&(left.clone() + "{"))
+                                    || args.contains(&(left.clone() + "["))
+                            }
+                            else
+                            {
+                                args.contains(&left)
+                            })
                     {
                         if let Some(r) = split.next()
                         {
-                            let l = l.chars().collect::<Vec<char>>();
-                            if !args.is_empty()
+                            let le = l.chars().collect::<Vec<char>>();
+                            if !options.interactive
                             {
-                                let mut blacklist = vec![left];
                                 get_file_vars(options, &mut vars, lines.clone(), r, &mut blacklist);
+                                if blacklist.contains(&l)
+                                {
+                                    continue;
+                                }
+                                blacklist.push(l);
                             }
                             for (i, v) in vars.iter().enumerate()
                             {
                                 if v.name.split(|c| c == &'(').next()
-                                    == l.split(|c| c == &'(').next()
-                                    && v.name.contains(&'(') == l.contains(&'(')
+                                    == le.split(|c| c == &'(').next()
+                                    && v.name.contains(&'(') == le.contains(&'(')
                                     && v.name.iter().filter(|c| c == &&',').count()
-                                        == l.iter().filter(|c| c == &&',').count()
+                                        == le.iter().filter(|c| c == &&',').count()
                                 {
                                     if r == "null"
                                     {
-                                        vars.remove(i);
+                                        if let Err(s) =
+                                            add_var(le, r, i, &mut vars, options, true, true, true)
+                                        {
+                                            println!("{}", s)
+                                        }
                                     }
-                                    else
+                                    else if let Err(s) =
+                                        add_var(le, r, i, &mut vars, options, true, true, false)
                                     {
-                                        add_var(l, r, i, &mut vars, options, true, true);
+                                        println!("{}", s)
                                     }
                                     continue 'upper;
                                 }
                             }
                             for (i, j) in vars.clone().iter().enumerate()
                             {
-                                if j.name.len() <= l.len()
+                                if j.name.len() <= le.len()
                                 {
-                                    add_var(l, r, i, &mut vars, options, false, false);
+                                    if let Err(s) =
+                                        add_var(le, r, i, &mut vars, options, false, false, false)
+                                    {
+                                        println!("{}", s)
+                                    }
                                     continue 'upper;
                                 }
                             }
-                            add_var(l, r, 0, &mut vars, options, false, false);
+                            if let Err(s) =
+                                add_var(le, r, 0, &mut vars, options, false, false, false)
+                            {
+                                println!("{}", s)
+                            }
                         }
                     }
                 }
             }
         }
     }
-    let (mut file, mut unmod_lines) = if args.is_empty()
+    let (mut file, mut unmod_lines) = if options.interactive
     {
         #[cfg(unix)]
         let file_path = &(var("HOME").unwrap() + "/.config/kalc.history");
@@ -369,7 +420,6 @@ fn main()
     };
     let mut handles: Vec<JoinHandle<()>> = Vec::new();
     let mut cut: Vec<char> = Vec::new();
-    let cli = !args.is_empty();
     'main: loop
     {
         let mut input = Vec::new();
@@ -388,40 +438,51 @@ fn main()
             input = args.remove(0).chars().collect();
             let output;
             let funcvar;
-            (output, funcvar, graphable, varcheck) = match input_var(
-                &input.iter().map(convert).collect::<String>(),
-                vars.clone(),
-                &mut Vec::new(),
-                &mut 0,
-                options,
-                false,
-                &mut (false, 0, 0),
-                false,
-                0,
-                Vec::new(),
-            )
+            if input.contains(&'#')
             {
-                Ok(f) => f,
-                Err(s) =>
-                {
-                    println!("{}: {}", input.iter().collect::<String>(), s);
-                    continue;
-                }
-            };
-            if !graphable && !varcheck
-            {
-                if let Ok(n) = do_math(output, options, funcvar)
-                {
-                    print_answer(n, options, &colors);
-                }
+                graphable = true
             }
-            if let Some(time) = watch
+            else
             {
-                println!(" {}", time.elapsed().as_nanos());
-            }
-            else if !graphable && !varcheck
-            {
-                println!();
+                (output, funcvar, graphable, varcheck) = match input_var(
+                    &input.iter().map(convert).collect::<String>(),
+                    vars.clone(),
+                    &mut Vec::new(),
+                    &mut 0,
+                    options,
+                    false,
+                    false,
+                    0,
+                    Vec::new(),
+                )
+                {
+                    Ok(f) => f,
+                    Err(s) =>
+                    {
+                        println!("{}: {}", input.iter().collect::<String>(), s);
+                        continue;
+                    }
+                };
+                if !graphable && !varcheck
+                {
+                    match do_math(output, options, funcvar)
+                    {
+                        Ok(n) => print_answer(n, options, &colors),
+                        Err(s) =>
+                        {
+                            println!("{}: {}", input.iter().collect::<String>(), s);
+                            continue;
+                        }
+                    }
+                }
+                if let Some(time) = watch
+                {
+                    println!(" {}", time.elapsed().as_nanos());
+                }
+                else if !graphable && !varcheck
+                {
+                    println!();
+                }
             }
         }
         else
@@ -464,10 +525,10 @@ fn main()
                         {
                             end = input.len()
                         }
-                        if ((!options.real_time_output && c != '\x14')
-                            || (long && !slow)
-                            || (slow && c != '\x14'))
+                        if (!options.real_time_output || long || slow)
                             && !input.is_empty()
+                            && c != '\x14'
+                            && c != '\x06'
                         {
                             (frac, graphable, _, varcheck) = print_concurrent(
                                 &input,
@@ -501,7 +562,7 @@ fn main()
                             terminal::disable_raw_mode().unwrap();
                             std::process::exit(0);
                         }
-                        println!();
+                        print!("\n\x1b[G\x1b[K");
                         if c == '\x09'
                         {
                             graphable = false;
@@ -549,9 +610,9 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                    "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                    input.len()+if options.prompt{2}else{0}
-                                );
+                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        input.len() + if options.prompt { 2 } else { 0 }
+                                    );
                             }
                         }
                         else
@@ -564,14 +625,12 @@ fn main()
                             print!(
                                 " {}{}",
                                 time,
-                                "\x08".repeat(
-                                    time.to_string().len() + 1 + end - start - (placement - start)
-                                )
+                                "\x08".repeat(time.to_string().len() + 1 + end - placement)
                             );
                         }
                         else
                         {
-                            print!("{}", "\x08".repeat(end - start - (placement - start)));
+                            print!("{}", "\x08".repeat(end - placement));
                         }
                     }
                     '\x7F' =>
@@ -614,9 +673,9 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                    "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                    input.len()+if options.prompt{2}else{0}
-                                );
+                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        input.len() + if options.prompt { 2 } else { 0 }
+                                    );
                             }
                         }
                         else
@@ -629,14 +688,12 @@ fn main()
                             print!(
                                 " {}{}",
                                 time,
-                                "\x08".repeat(
-                                    time.to_string().len() + 1 + end - start - (placement - start)
-                                )
+                                "\x08".repeat(time.to_string().len() + 1 + end - placement)
                             );
                         }
                         else
                         {
-                            print!("{}", "\x08".repeat(end - start - (placement - start)));
+                            print!("{}", "\x08".repeat(end - placement));
                         }
                     }
                     '\x11' =>
@@ -670,9 +727,9 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                    "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                    input.len()+if options.prompt{2}else{0}
-                                );
+                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        input.len() + if options.prompt { 2 } else { 0 }
+                                    );
                             }
                         }
                         else
@@ -702,16 +759,16 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                    "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                    input.len()+if options.prompt{2}else{0}
-                                );
+                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        input.len() + if options.prompt { 2 } else { 0 }
+                                    );
                             }
                         }
                         else
                         {
                             clearln(&input, start, end, options, &colors);
                         }
-                        print!("{}", "\x08".repeat(end - start - (placement - start)));
+                        print!("{}", "\x08".repeat(end - placement));
                     }
                     '\x19' =>
                     {
@@ -734,16 +791,16 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                    "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                    input.len()+if options.prompt{2}else{0}
-                                );
+                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        input.len() + if options.prompt { 2 } else { 0 }
+                                    );
                             }
                         }
                         else
                         {
                             clearln(&input, start, end, options, &colors);
                         }
-                        print!("{}", "\x08".repeat(end - start - (placement - start)));
+                        print!("{}", "\x08".repeat(end - placement));
                     }
                     '\x17' =>
                     {
@@ -768,16 +825,16 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                    "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                    input.len()+if options.prompt{2}else{0}
-                                );
+                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        input.len() + if options.prompt { 2 } else { 0 }
+                                    );
                             }
                         }
                         else
                         {
                             clearln(&input, start, end, options, &colors);
                         }
-                        print!("{}", "\x08".repeat(end - start - (placement - start)));
+                        print!("{}", "\x08".repeat(end - placement));
                     }
                     '\x16' =>
                     {
@@ -802,16 +859,16 @@ fn main()
                                 {
                                     slow = true;
                                     print!(
-                                    "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                    input.len()+if options.prompt{2}else{0}
-                                );
+                                            "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                            input.len() + if options.prompt { 2 } else { 0 }
+                                        );
                                 }
                             }
                             else
                             {
                                 clearln(&input, start, end, options, &colors);
                             }
-                            print!("{}", "\x08".repeat(end - start - (placement - start)));
+                            print!("{}", "\x08".repeat(end - placement));
                         }
                     }
                     '\x15' =>
@@ -833,16 +890,16 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                    "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                    input.len()+if options.prompt{2}else{0}
-                                );
+                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        input.len() + if options.prompt { 2 } else { 0 }
+                                    );
                             }
                         }
                         else
                         {
                             clearln(&input, start, end, options, &colors);
                         }
-                        print!("{}", "\x08".repeat(end - start - (placement - start)));
+                        print!("{}", "\x08".repeat(end - placement));
                     }
                     '\x10' =>
                     {
@@ -874,16 +931,16 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                    "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                    input.len()+if options.prompt{2}else{0}
-                                );
+                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        input.len() + if options.prompt { 2 } else { 0 }
+                                    );
                             }
                         }
                         else
                         {
                             clearln(&input, start, end, options, &colors);
                         }
-                        print!("{}", "\x08".repeat(end - start - (placement - start)));
+                        print!("{}", "\x08".repeat(end - placement));
                     }
                     '\x1D' | '\x05' =>
                     {
@@ -922,9 +979,9 @@ fn main()
                             if slow
                             {
                                 print!(
-                                    "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                    input.len()+if options.prompt{2}else{0}
-                                );
+                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        input.len() + if options.prompt { 2 } else { 0 }
+                                    );
                             }
                         }
                         else
@@ -973,9 +1030,9 @@ fn main()
                             if slow
                             {
                                 print!(
-                                    "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                    input.len()+if options.prompt{2}else{0}
-                                );
+                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        input.len() + if options.prompt { 2 } else { 0 }
+                                    );
                             }
                         }
                         else
@@ -996,7 +1053,7 @@ fn main()
                                 end = input.len()
                             }
                             clearln(&input, start, end, options, &colors);
-                            print!("{}", "\x08".repeat(end - start - (placement - start)))
+                            print!("{}", "\x08".repeat(end - placement))
                         }
                         else if placement != 0
                         {
@@ -1052,7 +1109,7 @@ fn main()
                                 end -= start - placement;
                                 start = start - (start - placement);
                                 clearln(&input, start, end, options, &colors);
-                                print!("{}", "\x08".repeat(end - start - (placement - start)));
+                                print!("{}", "\x08".repeat(end - placement));
                             }
                             else if placement == s
                             {
@@ -1147,9 +1204,9 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                    "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                    input.len()+if options.prompt{2}else{0}
-                                );
+                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        input.len() + if options.prompt { 2 } else { 0 }
+                                    );
                             }
                         }
                         else
@@ -1162,14 +1219,12 @@ fn main()
                             print!(
                                 " {}{}",
                                 time,
-                                "\x08".repeat(
-                                    time.to_string().len() + 1 + end - start - (placement - start)
-                                )
+                                "\x08".repeat(time.to_string().len() + 1 + end - placement)
                             );
                         }
                         else if placement != input.len()
                         {
-                            print!("{}", "\x08".repeat(end - start - (placement - start)));
+                            print!("{}", "\x08".repeat(end - placement));
                         }
                     }
                 }
@@ -1190,15 +1245,11 @@ fn main()
             if !varcheck
             {
                 print!(
-                    "\x1b[G\x1b[K{}{}",
+                    "{}{}",
                     prompt(options, &colors),
                     if options.color { "\x1b[0m" } else { "" }
                 );
             }
-            else
-            {
-                print!("\x1b[G\x1b[K")
-            }
             stdout.flush().unwrap();
             if input.is_empty() && !varcheck
             {
@@ -1219,14 +1270,30 @@ fn main()
         }
         if varcheck
         {
-            set_commands_or_vars(
-                &mut colors,
-                &mut options,
-                Some(&mut stdout),
-                &mut vars,
-                cli,
-                &input,
-            );
+            if let Err(s) = set_commands_or_vars(&mut colors, &mut options, &mut vars, &input)
+            {
+                if !s.is_empty()
+                {
+                    print!("\x1b[G\x1b[K{}\x1b[B\x1b[G{}", s, prompt(options, &colors));
+                }
+                else
+                {
+                    print!(
+                        "{}{}",
+                        prompt(options, &colors),
+                        if options.color { "\x1b[0m" } else { "" }
+                    );
+                }
+            }
+            else
+            {
+                print!(
+                    "{}{}",
+                    prompt(options, &colors),
+                    if options.color { "\x1b[0m" } else { "" }
+                );
+            }
+            stdout.flush().unwrap()
         }
         else if options.graph && graphable
         {
@@ -1268,14 +1335,20 @@ fn main()
                                 );
                                 if s.contains('=')
                                 {
-                                    set_commands_or_vars(
+                                    if let Err(s) = set_commands_or_vars(
                                         &mut colors,
                                         &mut options,
-                                        None,
                                         &mut vars,
-                                        true,
                                         &s.chars().collect::<Vec<char>>(),
-                                    );
+                                    )
+                                    {
+                                        print!(
+                                            "\x1b[A\x1b[G\x1b[K{}\x1b[B\x1b[G{}",
+                                            s,
+                                            prompt(options, &colors)
+                                        );
+                                        stdout.flush().unwrap()
+                                    }
                                 }
                             }
                         }
@@ -1288,7 +1361,6 @@ fn main()
                             &mut 0,
                             options,
                             false,
-                            &mut (false, 0, 0),
                             false,
                             0,
                             Vec::new(),
