@@ -55,7 +55,11 @@ pub fn print_concurrent(
                 );
                 if s.contains(&'=')
                 {
-                    set_commands_or_vars(&mut colors, &mut options, None, &mut vars, true, &s);
+                    if let Err(s) = set_commands_or_vars(&mut colors, &mut options, &mut vars, s)
+                    {
+                        handle_err(s, unmodified_input, options, &colors, start, end);
+                        return (0, false, false, false);
+                    }
                 }
             }
         }
@@ -125,7 +129,6 @@ pub fn print_concurrent(
         &mut 0,
         options,
         false,
-        &mut (false, 0, 0),
         false,
         0,
         Vec::new(),
@@ -149,6 +152,7 @@ pub fn print_concurrent(
             {
                 func.drain(0..=func.iter().position(|c| c == &'(').unwrap());
                 func.pop();
+                func.pop();
                 for i in func.split(|c| c == &',')
                 {
                     func_vars.push((-1, i.iter().collect()));
@@ -161,13 +165,12 @@ pub fn print_concurrent(
                 &mut 0,
                 options,
                 false,
-                &mut (false, 0, 0),
                 true,
                 0,
                 Vec::new(),
             )
             {
-                Ok(n) => parsed_to_string(&n.0, &options, &colors),
+                Ok(n) => parsed_to_string(n.0, n.1, &options, &colors),
                 _ => String::new(),
             };
             if out.is_empty()
@@ -258,14 +261,23 @@ pub fn print_concurrent(
                                 );
                                 if s.contains('=')
                                 {
-                                    set_commands_or_vars(
+                                    if let Err(s) = set_commands_or_vars(
                                         &mut colors,
                                         &mut options,
-                                        None,
                                         &mut vars,
-                                        true,
                                         &s.chars().collect::<Vec<char>>(),
-                                    );
+                                    )
+                                    {
+                                        handle_err(
+                                            s,
+                                            unmodified_input,
+                                            options,
+                                            &colors,
+                                            start,
+                                            end,
+                                        );
+                                        return (0, false, false, false);
+                                    }
                                 }
                             }
                         }
@@ -283,7 +295,7 @@ pub fn print_concurrent(
             out.pop();
             let no_col = no_col(&out, options.color);
             let wrap = no_col
-                .split('\n')
+                .split(|c| c == &'\n')
                 .map(|i| {
                     if i.is_empty()
                     {
@@ -930,7 +942,7 @@ pub fn print_concurrent(
         else
         {
             let mut len = 0;
-            for i in no_col(&frac_out, options.color).chars()
+            for i in no_col(&frac_out, options.color)
             {
                 len += 1;
                 if i == '\n'
@@ -944,7 +956,7 @@ pub fn print_concurrent(
                 }
             }
             len = 0;
-            for i in no_col(&output, options.color).chars()
+            for i in no_col(&output, options.color)
             {
                 len += 1;
                 if i == '\n'
